import matplotlib.pyplot as plt
import numpy as np
import scipy.integrate as si

# débit instantané ième piston pompe à n piston
def qin(i,n,theta):
    return max(0,np.sin(theta-2*np.pi*(i-1)/n))

# tableau de valeurs pour theta
theta=[]
for x in range(0,int(100*2*np.pi)):
    theta.append(x/100)


# tracé débit instantané 3ème piston pompe à 5 pistons
plt.figure(1)
y=[]
for x in range(0,int(100*2*np.pi)):
    y.append(qin(3,5,x/100))
plt.plot(theta,y)
plt.title("3ème piston pompe à 5 pistons")
plt.ylabel('qin(3,5,theta)')
plt.xlabel("theta")


# débit global instantané pompe à n pistons
def Qn(n,theta):
    r=0
    for i in range(1,n+1):
        r=r+qin(i,n,theta)
    return r

# tracé pompe à 5 pistons
plt.figure(2)
y=[]
for i in range(6):
    y.append([])
for i in range(5):
    for x in range(0,int(100*2*np.pi)):
        y[i].append(qin(i+1,5,x/100))
    plt.plot(theta,y[i],label="piston "+str(i+1))
for x in range(0,int(100*2*np.pi)):
    y[5].append(Qn(5,x/100))
plt.plot(theta,y[5],label="débit global")
plt.xlabel("theta")
plt.title("pompe à 5 pistons")
plt.legend()

# tracé débit global pompe 1 à 10 pistons
plt.figure(3)
y=[]
for i in range(10):
    y.append([])
for i in range(10):
    for x in range(0,int(100*2*np.pi)):
        y[i].append(Qn(i+1,x/100))
    plt.plot(theta,y[i],label=str(i+1))
plt.xlabel("theta")
plt.title("Débit global pompe 1 à 10 pistons")
plt.legend()

# période du débit global
def periode(n):
    if n==1 :
        return 2*np.pi
    elif n%2==0:
        return 2*np.pi/n
    else :
        return 2*np.pi/(2*n)
    
# Valeur moyenne du débit
def Qmoy(n):
    T=periode(n)
    f = lambda x: Qn(n,x)
    return 1/T*si.quad(f,0,T)[0]

# Coefficient d'irrégularité
def delta(n):
    m=Qmoy(n)
    if n==1 :
        r=(Qn(n,np.pi/2)-m)/m
    else :
        r=(m-Qn(n,0))/m
    return r

# tracé coefficient irrégularité pompe 1 à 10 pistons
plt.figure(4)
x=[]
y=[]
for i in range(10):
    x.append(i+1)
    y.append(delta(i+1))
plt.plot(x,y,marker="o")
plt.xlabel("nombre de pistons")
plt.ylabel("delta")
plt.title("Coefficient d'irrégularité pompe 1 à 10 pistons")


# Affichage des tracés
plt.show()
